package no.bbs.bankid.bidcserver;

import no.bbs.bankid.bidcserver.loaders.InJarLoader;
import no.bbs.bankid.bidcserver.loaders.JavaLibraryPathLoader;
import no.bbs.bankid.bidcserver.loaders.Loader;
import no.bbs.bankid.bidcserver.loaders.LoaderExecutor;

import java.util.LinkedHashSet;
import java.util.Set;

/**
 * C Server Library for Bank Id.
 * It contains just single method which loads library into jvm.
 * It uses different approaches, next is used only if previous fails.
 *
 * Created by lkale on 27.10.2015.
 */
public class CServerLibrary {

    private static Set<Loader> loaders = new LinkedHashSet<Loader>();
    static {
        LoaderExecutor loaderExecutor = new LoaderExecutor();
        // Add more loaders if you wish. order matters :)
        loaders.add(new InJarLoader(loaderExecutor));
        loaders.add(new JavaLibraryPathLoader(loaderExecutor));
    }

    public boolean load() {
        for(Loader loader : loaders) {
            StringBuilder messages = new StringBuilder();
            boolean loaded = loader.load(messages);
            if (loaded) {
                return true;
            } else {
                System.out.print(loader.getClass().getSimpleName() + ":");
                System.out.print(messages.toString());
            }
        }
        return false;
    }

    public static boolean activate() {
        return new CServerLibrary().load();
    }
}
