package no.bbs.bankid.bidcserver.loaders;

import java.io.File;
import java.lang.reflect.Field;
import java.nio.file.Paths;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * This class loads library from package inside jar file native c libraries.
 *
 * Created by lkale on 27.10.2015.
 */
public class InJarLoader implements Loader {

    private final LoaderExecutor loaderExecutor;
    private final PackageClibResolver packageClibResolver;
    private final PackageClibExtractor packageClibExtractor;

    public InJarLoader(LoaderExecutor loaderExecutor) {
        this.loaderExecutor = loaderExecutor;
        this.packageClibResolver = new PackageClibResolver();
        this.packageClibExtractor = new PackageClibExtractor();
    }

    @Override
    public boolean load(StringBuilder messages) {
        Set<String> libs = packageClibResolver.listLibs();
        String outLibDirectory = packageClibExtractor.findOutLibDirectory();
        //System.out.println("BID C Server libraries will be installed in: " + outLibDirectory);
        for(String inJarFile : libs) {
            packageClibExtractor.extractFromJar(inJarFile, outLibDirectory, messages);
        }
        String capi = outLibDirectory + File.separator + System.mapLibraryName("CAPI");
        return loaderExecutor.load(capi, messages);
    }

    private static void alterLibraryPathInRuntime(String directory) {
        String libraryPath = System.getProperty("java.library.path");
        System.out.println(libraryPath);
        libraryPath += ";" + directory;
        System.setProperty("java.library.path", libraryPath);

        //set sys_paths to null
        Field sysPathsField = null;
        try {
            sysPathsField = ClassLoader.class.getDeclaredField("sys_paths");
        } catch (NoSuchFieldException e) {
            return;
        }

        sysPathsField.setAccessible(true);
        try {
            sysPathsField.set(System.class.getClassLoader(), null);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }
}
