package no.bbs.bankid.bidcserver.loaders;

import java.io.File;
import java.util.Set;

/**
 * Class which executes loading library.
 *
 * Created by lkale on 27.10.2015.
 */
public class LoaderExecutor {

    static final String FILE_SEPARATOR;
    static final String LINE_SEPARATOR;

    static {
        LINE_SEPARATOR = System.getProperty("line.separator");
        FILE_SEPARATOR = System.getProperty("file.separator");
    }

    boolean load(Set<String> libNames, StringBuilder message) {
        for(String libName : libNames) {
            if (!load(libName, message)) {
                return false;
            }
        }
        return true;
    }

    boolean load(String libName, StringBuilder message) {
        try {
            if(libName.contains(FILE_SEPARATOR)) {
                System.load(libName);
            } else {
                System.loadLibrary(libName);
            }
            return true;
        } catch (UnsatisfiedLinkError e) {
            if (message.length() == 0) {
                message.append(LINE_SEPARATOR);
            }
            message.append('\t');
            message.append(e.getMessage());
            message.append(LINE_SEPARATOR);
        }
        return false;
    }
}
