package no.bbs.bankid.bidcserver.loaders;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Paths;

/**
 * Extracting libraries from jar.
 *
 * Created by lkale on 27.10.2015.
 */
public class PackageClibExtractor {

    public String findOutLibDirectory() {
        // Working directory.
        return Paths.get("").toAbsolutePath().toString();
    }

    public boolean extractFromJar(String inJarFile, String outDirectory, StringBuilder message) {
        String outFileName = new File(inJarFile).getName();

        FileOutputStream os = null;
        InputStream is = null;
        File file = new File(outDirectory, outFileName);
        boolean extracted = false;

        try {
            if (file.exists()) {
                if (!file.delete()) {
                    System.out.println("Deleting fails: " + inJarFile);
                }
            }

            if (!file.exists()) {
                //System.out.println("Extracting: " + inJarFile);
                is = getClass().getClassLoader().getResourceAsStream(inJarFile);
                if (is != null) {
                    extracted = true;

                    // Copy file
                    int read;
                    byte [] buffer = new byte[4096];
                    os = new FileOutputStream(file);
                    while ((read = is.read (buffer)) != -1) {
                        os.write(buffer, 0, read);
                    }
                    os.close ();
                    is.close ();

                    chmod("755", file.getAbsolutePath());
                } else {
                    System.out.println("No InputStream found at: " + inJarFile);
                    return false;
                }
            }
            return true;
        } catch(Throwable e) {
            try {
                if (os != null) {
                    os.close ();
                }
            } catch (IOException e1) {
                // ignore
            }
            try {
                if (is != null) {
                    is.close ();
                }
            } catch (IOException e1) {
                // ignore
            }
            if (extracted && file.exists()) {
                // something goes wrong, so we do not need file.
                file.delete();
            }
            return false;
        }
    }


    private void chmod(String permission, String path) {
        if (PackageClibResolver.Os.windows.isCurrent()) {
            return;
        }
        try {
            String[] command = new String []{"chmod", permission, path};
            Runtime.getRuntime().exec(command).waitFor();
        } catch (Throwable e) {
            // ignore
        }
    }
}
